use crate::spotify::ContentKind;
use crate::{spotify, youtube};
use chrono::{Date, Utc};
use std::time::Duration;

pub(crate) enum MusicData {
    Track(Track),
    Album(Album),
}

#[derive(Debug, Clone)]
pub(crate) struct Track {
    name: String,
    authors: Vec<Author>,
    duration: Duration,
    album: Option<Album>,
    description: Option<String>,
    lyrics: Option<String>,
}

#[derive(Debug, Clone)]
pub(crate) struct Album {
    name: String,
    authors: Vec<Author>,
    description: Option<String>,
    year: Option<Date<Utc>>,
}

#[derive(Debug, Clone)]
pub(crate) struct Author {
    name: String,
}

// The enum holds all the currently supported type of Id which the engine can search for
pub(crate) enum ServiceIdKind {
    Spotify(String),
    Youtube(String),
    Automatic(String),
}

// This struct will allow us in the future to search, cache and store data and metadata regarding
// tracks, albums and playlists
pub(crate) struct MusicEngine {
    spotify: spotify::Client,
    youtube: youtube::Client,
}

impl MusicEngine {
    pub(crate) async fn new() -> Self {
        MusicEngine {
            spotify: spotify::Client::new().await,
            youtube: youtube::Client::new().await,
        }
    }

    pub(crate) fn new_with_dependencies(
        spotify_client: spotify::Client,
        youtube_client: youtube::Client,
    ) -> Self {
        MusicEngine {
            spotify: spotify_client,
            youtube: youtube_client,
        }
    }

    pub(crate) async fn search_by_name(&self, name: &str) {
        todo!("In the future it would be possible to search for all metadata on a record from this call")
    }

    pub(crate) async fn search_by_id(&self, id: ServiceIdKind) -> Option<MusicData> {
        match id {
            ServiceIdKind::Spotify(id) => {
                let entry_kind = spotify::get_entry_kind(id.as_str());
                match entry_kind {
                    None => None,
                    Some(entry) => match entry {
                        ContentKind::Track(id) => {
                            self.spotify.get_track(id.as_str()).await.map(|track| {
                                MusicData::Track(Track {
                                    name: track.name.clone(),
                                    authors: track
                                        .artists
                                        .iter()
                                        .map(|artist| Author {
                                            name: artist.clone(),
                                        })
                                        .collect(),
                                    duration: track.duration.clone(),
                                    album: None,       // FIXME missing metadata
                                    description: None, // FIXME missing metadata
                                    lyrics: None,      // FIXME missing metadata
                                })
                            })
                        }
                        ContentKind::Album(id) => {
                            self.spotify.get_album(id.as_str()).await.map(|album| {
                                MusicData::Album(Album {
                                    name: album.name.clone(),
                                    authors: album
                                        .artists
                                        .iter()
                                        .map(|artist| Author {
                                            name: artist.clone(),
                                        })
                                        .collect(),
                                    description: None, // FIXME missing metadata
                                    year: None,        // FIXME missing metadata
                                })
                            })
                        }
                        _ => None,
                    },
                }
            }
            _ => todo!("Search type not present yet"), //TODO implement
        }
    }
}
